/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

/**
 * @author Bruno Neves
 *
 */
public class RelatorioVendasBilheteiroSintetico extends Relatorio {


	public RelatorioVendasBilheteiroSintetico(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new ArrayDataSource(this) {
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();

				StringBuilder sql = new StringBuilder();
				
				sql.append("select ");
				sql.append("    pv.NUMPUNTOVENTA, us.cveusuario , us.NOMBUSUARIO, tpv.DESCTIPO,");
				sql.append("    sum (case when ca.motivocancelacion_id is null then 1 else 0 end) qtdVendas, ");
				sql.append("    sum (case when ca.motivocancelacion_id is null then ");
				sql.append("        (coalesce(ca.preciopagado,0) ");
				sql.append("        + coalesce(ca.importepedagio,0) ");
				sql.append("        + coalesce(ca.importetaxaembarque,0) ");
				sql.append("        + coalesce(ca.importeseguro,0) ) else 0 end ) vlrVendas, ");
				sql.append("    sum (case when ca.motivocancelacion_id is not null then 1 else 0 end) qtdCancelados, ");
				sql.append("    sum (case when ca.motivocancelacion_id is not null then ");
				sql.append("        (coalesce(ca.preciopagado,0) ");
				sql.append("        + coalesce(ca.importepedagio,0) ");
				sql.append("        + coalesce(ca.importetaxaembarque,0) ");
				sql.append("        + coalesce(ca.importeseguro,0) ) else 0 end ) vlrCancelados ");
				sql.append("from ");
				sql.append("    caja ca ");
				sql.append("    inner join usuario us on us.usuario_id = ca.usuario_id ");
				sql.append("    inner join punto_venta pv on pv.puntoventa_id = ca.puntoventa_id ");
				sql.append("    INNER JOIN  TIPO_PTOVTA tpv ON tpv.TIPOPTOVTA_ID = pv.TIPOPTOVTA_ID ");
				sql.append("where ");
				sql.append("    ca.indreimpresion = 0 ");
				sql.append(" and ca.feccorte >= :DATA_INICIAL ");
				sql.append(" and ca.feccorte <= :DATA_FINAL ");
				if(parametros.get("EMPRESA_ID") != null){
					sql.append(" and ca.EMPRESAPUNTOVENTA_ID = :EMPRESA_ID");
				}
				
				if (parametros.get("NUMPUNTOVENTA") != null) {
					sql.append(" and ca.puntoventa_id IN (" + parametros.get("NUMPUNTOVENTA").toString() + ")");
				}
				if(parametros.get("TIPOPUNTOVENTA_ID") != null && !parametros.get("TIPOPUNTOVENTA_ID").toString().equals("-1") ){
					sql.append(" and pv.TIPOPTOVTA_ID  = :TIPOPUNTOVENTA_ID ");
				}
				sql.append(" group by pv.NUMPUNTOVENTA, us.cveusuario, us.NOMBUSUARIO , tpv.DESCTIPO ");
				sql.append(" order by pv.NUMPUNTOVENTA, us.NOMBUSUARIO ");
				

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql.toString());
				if(parametros.get("EMPRESA_ID") != null){
					stmt.setInt("EMPRESA_ID", Integer.valueOf(parametros.get("EMPRESA_ID").toString()));
				}
				
				stmt.setTimestamp("DATA_INICIAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
				stmt.setTimestamp("DATA_FINAL", new Timestamp(DateUtil.fimFecha((Date) parametros.get("DATA_FINAL")).getTime()));
				if(parametros.get("TIPOPUNTOVENTA_ID") != null && !parametros.get("TIPOPUNTOVENTA_ID").toString().equals("-1") ){
					stmt.setInt("TIPOPUNTOVENTA_ID", Integer.valueOf(parametros.get("TIPOPUNTOVENTA_ID").toString()));
				}
				
				ResultSet rset = stmt.executeQuery();

				while (rset.next()) {

					Map<String, Object> dataResult = new HashMap<String, Object>();

					dataResult.put("NUMPUNTOVENTA", rset.getString("NUMPUNTOVENTA"));
					dataResult.put("CVEUSUARIO", rset.getString("CVEUSUARIO"));
					dataResult.put("NOMBUSUARIO", rset.getString("NOMBUSUARIO"));
					dataResult.put("QTDVENDAS", rset.getInt("QTDVENDAS"));
					dataResult.put("VLRVENDAS", rset.getBigDecimal("VLRVENDAS"));
					dataResult.put("QTDCANCELADOS", rset.getInt("QTDCANCELADOS"));
					dataResult.put("VLRCANCELADOS", rset.getBigDecimal("VLRCANCELADOS"));
					dataResult.put("DESCTIPO", rset.getString("DESCTIPO"));
					this.dados.add(dataResult);

				}

				this.resultSet = rset;
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio#processaParametros()
	 */
	@Override
	protected void processaParametros() throws Exception {

	}

}
